/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *    Author : Boris Mikhailenko <stg34@ua.fm>
 */

 /*
 $Revision: 1.16 $
 $Date: 2007/03/15 08:37:01 $
 */

#ifndef common_h
#define common_h

#include <time.h>
#include <string>
#include <limits.h>

#ifdef LINUX
#include <stdint.h>
#endif

#ifdef FREE_BSD5
#include <inttypes.h>
#endif

#ifdef FREE_BSD
#include <sys/inttypes.h>
#endif

#include "blowfish.h"
#include "stg_const.h"

#ifndef ENODATA
#define ENODATA 61
#endif

#ifndef EBADMSG
#define EBADMSG 74
#endif

#define STAT_TIME_3         (1)
#define STAT_TIME_2         (2)
#define STAT_TIME_1         (3)
#define STAT_TIME_1_2       (4)
#define STAT_TIME_1_4       (5)
#define STAT_TIME_1_6       (6)

#define FN_STR_LEN      (NAME_MAX)

#define SYS_IFACE_LEN   (9)

#define MAX(a, b) ((a) > (b) ? (a) : (b))

#define ST_F    0
#define ST_B    1
#define ST_KB   2
#define ST_MB   3

//using namespace std;

//-----------------------------------------------------------------------------
//typedef char string16[DIR_NAME_LEN];

//-----------------------------------------------------------------------------
struct ia_version
{
    short minor;
    short major;
    short proto;
};
//-----------------------------------------------------------------------------
const char    * IntToKMG(long long a, int statType = ST_F);
const char    * LogDate(time_t t);
int             ParesTimeStat(const char * str);
int             IsTimeStat(struct tm * t, int statTime);
bool            IsDigit(char c);
bool            IsAlpha(char c);
int             strtodouble2(const char * s, double &a);
int             printfd(const char * __file__, const char * fmt, ...);
void            Encode12(char * dst, const char * src, int srcLen);
void            Decode21(char * dst, const char * src);
int             ParseIPString(const char * str, uint32_t * ips, int maxIP);
void            KOIToWin(const char * s1, char * s2, int l);
void            WinToKOI(const char * s1, char * s2, int l);
void            KOIToWin(const std::string & s1, std::string * s2);
void            WinToKOI(const std::string & s1, std::string * s2);
int             DaysInMonth(int year, int mon);
int             DaysInCurrentMonth();
void            EnDecodeInit(const char * key, int passwdLen, BLOWFISH_CTX *ctx);
void            DecodeString(char * d, const char * s, BLOWFISH_CTX *ctx);
void            EncodeString(char * d, const char * s, BLOWFISH_CTX *ctx);
int             Min8(int a);
char          * inet_ntostr(unsigned long);
int             strprintf(std::string * str, const char * fmt, ...);
int             ParseTariffTimeStr(const char * str, int &h1, int &m1, int &h2, int &m2);
uint32_t        CalcMask(uint32_t msk);
#ifdef WIN32
void            EncodeStr(char * str, unsigned long serial, int useHDD);
void            DecodeStr(char * str, unsigned long serial, int useHDD);
#endif //WIN32

//-----------------------------------------------------------------------------
template <typename varT>
int str2x(const std::string & str, varT & x)
{
    int pos = 0;
    int minus = 1;

    if (str.empty())
        return -1;

    if (str[0] == '+')
        pos++;

    if (str[0] == '-')
    {
        pos++;
        minus = -1;
    }

    if ((str[pos] < '0' || str[pos] > '9'))
        return -1;

    x = str[pos++] - '0';

    for (unsigned i = pos; i < str.size(); i++)
    {
        if ((str[pos] < '0' || str[pos] > '9'))
            return -1;

        x *= 10;
        x += str[i] - '0';
    }

    x*= minus;

    return 0;
}
//-----------------------------------------------------------------------------
template <typename varT>
const std::string & x2str(varT x, std::string & s)
{
    varT xx = x;
    int pos = 1;

    x /= 10;
    while (x != 0)
    {
        x /= 10;
        pos++;
    }

    if (xx < 0)
    {
        pos++;
        s.resize(pos, 0);
        s[0] = '-';
    }
    else if (xx > 0)
    {
        s.resize(pos, 0);
    }
    else
    {
        s.resize(1, 0);
        s[0] = '0';
        return s;
    }

    x = xx;

    while (x != 0)
    {
        if (x < 0)
            s[--pos] = -(x % 10) + '0';
        else
            s[--pos] = x % 10 + '0';

        x /= 10;
    }

    return s;
}
//-----------------------------------------------------------------------------

#endif




