/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
Date: 18.09.2002
*/

/*
* Author : Boris Mikhailenko <stg34@stargazer.dp.ua>
*/
//---------------------------------------------------------------------------
#ifndef IA_AUTH_C_H
#define IA_AUTH_C_H

#ifndef WIN32
#include <netinet/in.h>
#include <pthread.h>
#else
#include <winsock2.h>
#endif

#include <string>
#include <vector>
#include <map>

#include "blowfish.h"
#include "ia_packets.h"

#define IA_BIND_ERROR           (1)
#define IA_SERVER_ERROR         (2)
#define IA_FCNTL_ERROR          (3)
#define IA_GETHOSTBYNAME_ERROR  (4)

using namespace std;

typedef void (*tpStatusChangedCb)(int status, void * data);
typedef void (*tpStatChangedCb)(const LOADSTAT & stat, void * data);
typedef void (*tpCallBackInfoFn)(const string & message, int infoType, int showTime, int sendTime, void * data);
typedef void (*tpCallBackErrorFn)(const string & message, int netError, void * data);
typedef void (*tpCallBackDirNameFn)(const vector<string> & dirName, void * data);

//---------------------------------------------------------------------------
class IA_CLIENT_PROT
{
#ifdef WIN32
friend unsigned long WINAPI RunW(void * data);
#else
friend void * RunL(void * data);
#endif

public:
    IA_CLIENT_PROT(const string & sn, uint16_t p);
    ~IA_CLIENT_PROT();

    void        Start();
    void        Stop();
    void        GetStat(LOADSTAT * ls);

    void        SetServer(const string & sn, unsigned short port);
    void        SetLogin(const string & login);
    void        SetPassword(const string & password);
    void        SetEnabledDirs(const bool * selectedDirs);

    void        SetStatusChangedCb(tpStatusChangedCb p, void * data);
    void        SetStatChangedCb(tpStatChangedCb p, void * data);
    void        SetInfoCb(tpCallBackInfoFn p, void * data);
    void        SetErrorCb(tpCallBackErrorFn p, void * data);
    void        SetDirNameCb(tpCallBackDirNameFn p, void * data);

    int         Connect();
    int         Disconnect();
    int         GetAuthorized() const;   //    ?
    int         GetPhase() const { return phase; };
    int         GetStatus() const;
    int         GetReconnect() const;
    void        SetReconnect(int r);
    char        GetProtoVer() const;
    void        GetMessageText(string * text) const;
    void        GetInfoText(string * text) const;
    int         GetStrError(string * error) const;

    void        SetProxyMode(bool on);
    bool        GetProxyMode();

    void        SetIP(uint32_t ip);
    uint32_t    GetIP();

private:
    void            Run();
    int             NetRecv();
    int             NetSend(int n);
    bool            GetNonstop() const { return nonstop; };
    void            PrepareNet();
    int             DeterminatePacketType(const char * buffer);

    int             Process_CONN_SYN_ACK_8(const char * buffer);
    int             Process_ALIVE_SYN_8(const char * buffer);
    int             Process_DISCONN_SYN_ACK_8(const char * buffer);
    int             Process_FIN_8(const char * buffer);
    int             Process_INFO_8(const char * buffer);
    int             Process_ERROR(const char * buffer);

    int             Prepare_CONN_SYN_8(char * buffer);
    int             Prepare_CONN_ACK_8(char * buffer);
    int             Prepare_ALIVE_ACK_8(char * buffer);
    int             Prepare_DISCONN_SYN_8(char * buffer);
    int             Prepare_DISCONN_ACK_8(char * buffer);

    void            FillHdr8(char * buffer, unsigned long ip);
    int             Send(char * buffer, int len);
    int             Recv(char * buffer, int len);

    LOADSTAT        stat;
    int             action;
    int             phase;
    int             phaseTime;                  //    
    string          messageText;                //   
    string          infoText;
    mutable string  strError;
    mutable int     codeError;
    bool            nonstop;
    bool            isNetPrepared;
    bool            proxyMode;

    BLOWFISH_CTX    ctxPass;
    BLOWFISH_CTX    ctxHdr;

    bool            selectedDirs[DIR_NUM];

    string          password;
    string          login;

    #ifdef WIN32
    WSADATA wsaData;
    #else
    pthread_t thread;
    #endif

    string          serverName;                     //  
    uint16_t        port;                           //  
    uint32_t        ip;                             // Proxy IP

    struct sockaddr_in  localAddrS;      //  
    struct sockaddr_in  localAddrR;      //  
    struct sockaddr_in  servAddr;        //  

    bool            firstConnect;
    int             reconnect;
    int             sockr;
    int             protNum;                    // ,     a
    int             userTimeout;
    int             aliveTimeout;
    unsigned int    rnd;

    tpStatusChangedCb   pStatusChangedCb;
    tpStatChangedCb     pStatChangedCb;
    tpCallBackInfoFn    pInfoCb;
    tpCallBackErrorFn   pErrorCb;
    tpCallBackDirNameFn pDirNameCb;

    void              * statusChangedCbData;
    void              * statChangedCbData;
    void              * infoCbData;
    void              * errorCbData;
    void              * dirNameCbData;

    map<string, int>    packetTypes;

    CONN_SYN_8        * connSyn8;
    CONN_SYN_ACK_8    * connSynAck8;
    CONN_ACK_8        * connAck8;
    ALIVE_SYN_8       * aliveSyn8;
    ALIVE_ACK_8       * aliveAck8;
    DISCONN_SYN_8     * disconnSyn8;
    DISCONN_SYN_ACK_8 * disconnSynAck8;
    DISCONN_ACK_8     * disconnAck8;
    ERR_8               err;
    INFO_8            * info;
};
//---------------------------------------------------------------------------
#ifdef WIN32
unsigned long WINAPI RunW(void *);
#else
void * RunW(void *);
#endif

//---------------------------------------------------------------------------
#endif //IA_AUTH_C_H


