/* $Id: stg_userlog.h,v 1.3 2006/11/02 14:38:12 aforizm Exp $ */

#ifndef LOG_H
#define LOG_H

#ifndef _XOPEN_SOURCE
#define _XOPEN_SOURCE
#endif


#include <time.h>
#include <pcre.h>

#include <string>
#include <vector>

#ifdef LINUX
#include <stdint.h>
#endif

#ifdef FREE_BSD5
#include <inttypes.h>
#endif

#ifdef FREE_BSD
#include <sys/inttypes.h>
#endif

#include "stg_const.h"


using namespace std;

//#define MAX_COMMON_LOG_STR_SIZE (1024)
#define STR_DATE_TIME_FORMAT ("%Y-%m-%d %H:%M:%S")

struct PARAM_INFO_BIN
{
    PARAM_INFO_BIN()
    {
        paramName = "";
        oldValue = 0.0;
        newValue = 0.0;
        date = (time_t)0;
        admin = "";
        ip = "";
        user = "";
    }
    string user;
    string paramName;
    double oldValue;
    double newValue;
    time_t date;
    string admin;
    string ip;
    string comment;
};

struct PARAM_INFO_TEXT
{
    PARAM_INFO_TEXT()
    {
        paramName = "";
        oldValue = "";
        newValue = "";
        date = (time_t)0;
        admin = "";
        ip = "";
        user = "";
    }
    string user;
    string paramName;
    string oldValue;
    string newValue;
    time_t date;
    string admin;
    string ip;
    string comment;
};

struct CON_DISCON_INFO
{
        CON_DISCON_INFO()
        {
            cDate = (time_t)0;
            dDate = (time_t)0;
            cash = 0;
            ip = "";
            user = "";
        }
        string user;
        time_t cDate;
        string ip;
        time_t dDate;
        double cash;
        uint64_t sessionUp[DIR_NUM];
        uint64_t sessionDown[DIR_NUM];
        uint64_t monthUp[DIR_NUM];
        uint64_t monthDown[DIR_NUM];
};

int operator<(const struct PARAM_INFO_TEXT & info1, const struct PARAM_INFO_TEXT & info2);
int operator<(const struct CON_DISCON_INFO & info1, const struct CON_DISCON_INFO & info2);

class LOG_PARSER
{
    private:
        string userDataPath;
        string userName;
        time_t fromDt;
        time_t tillDt;
        vector<struct CON_DISCON_INFO> conDisconInfo;
        vector<struct PARAM_INFO_TEXT> paramInfoText;
        vector<struct PARAM_INFO_BIN> paramInfoBin;
        vector<string> notRecognize;
        pcre * pcreConnect;
        pcre * pcreTime;
        pcre * pcreDisconnect0;
        pcre * pcreDisconnect1;
        pcre * pcreDisconnect2;
        pcre * pcreDisconnect3;
        pcre * pcreDisconnect4;
        pcre * pcreParameter0;
        pcre * pcreParameter1;
        pcre * pcreParameter2;
        pcre * pcreParameter3;

        int CompilePatterns();
        time_t Str2TimeT(const char * dTime, const char * dTimeFormat);
        string GetLogPath();
        void SetUnknownConDisconInfo(struct CON_DISCON_INFO & data);
        void CopyConnectInfo(struct CON_DISCON_INFO & dest, const struct CON_DISCON_INFO & src);
        int ParamInfoText2Bin(const struct PARAM_INFO_TEXT & paramInfoT, struct PARAM_INFO_BIN & paramInfoB);
        int ParseEventTime(const string & logStr, time_t & dTime);
        int ParseConnectStr(const string & logStr, struct CON_DISCON_INFO & data);
        int ParseSingleDisconnectStr(const string & logStr, struct CON_DISCON_INFO & data);
        int ParseFirstDisconnectStr(const string & logStr, struct CON_DISCON_INFO & data);
        int ParseSecondDisconnectStr(const string & logStr, struct CON_DISCON_INFO & data);
        int ParseThirdDisconnectStr(const string & logStr, struct CON_DISCON_INFO & data);
        int ParseParamStr(const string & logStr, struct PARAM_INFO_TEXT & paramInfo);
    public:
        LOG_PARSER();
        ~LOG_PARSER();

        void SetUserDataPath(const char * path);
        void SetUserName(const char * name);
        void SetTimeFrom(time_t dTime);
        void SetTimeFrom(const char * dTime, const char * dTimeFormat);
        void SetTimeTill(time_t dTime);
        void SetTimeTill(const char * dTime, const char * dTimeFormat);
        time_t GetTimeFrom();
        time_t GetTimeTill();

        const vector<struct CON_DISCON_INFO> & GetConDisconInfo();
        const vector<struct PARAM_INFO_TEXT> & GetParamInfoText();
        const vector<struct PARAM_INFO_BIN> & GetParamInfoBin();
        const vector<string> & GetNotRecognize();
        int ParseLog();
};

#endif
