 /*
 $Revision: 1.13 $
 $Date: 2007/03/15 09:41:56 $
 */

#ifndef USER_IPS_H
#define USER_IPS_H

#ifdef LINUX
#include <stdint.h>
#endif

#ifdef FREE_BSD5
#include <inttypes.h>
#endif

#ifdef FREE_BSD
#include <sys/inttypes.h>
#endif

#include <stdio.h>
#include <vector>
#include <algorithm>
#include <string>
#include <iostream>
#include <sstream>
/////////////////////////
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "user_ips.h"
#include "common.h"

using namespace std;

//-------------------------------------------------------------------------
struct IP_MASK
{
IP_MASK() { ip = 0; mask = 0; }
IP_MASK(const IP_MASK & ipm)  { ip = ipm.ip; mask = ipm.mask; }
uint32_t ip;
uint32_t mask;
};
//-------------------------------------------------------------------------
class USER_IPS
{
    friend ostream & operator<< (ostream & o, const USER_IPS & i);
    friend stringstream & operator<< (stringstream & s, const USER_IPS & i);
    friend const USER_IPS StrToIPS(const string & ipsStr) throw(string);

public:
    USER_IPS();
    USER_IPS(const USER_IPS &);
    USER_IPS & operator=(const USER_IPS &);
    const IP_MASK & operator[](int idx) const;
    string GetIpStr() const;
    bool IsIPInIPS(uint32_t ip) const;
    bool OnlyOneIP() const;
    int  Count() const;

private:
    uint32_t CalcMask(unsigned int msk) const;
    vector<IP_MASK> ips;
};
//-------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline USER_IPS::USER_IPS()
{
ips.clear();
}
//-----------------------------------------------------------------------------
inline USER_IPS::USER_IPS(const USER_IPS & i)
{
if (this == &i)
    return;

ips.clear();
copy(i.ips.begin(), i.ips.end(), back_inserter(ips));
}
//-----------------------------------------------------------------------------
inline USER_IPS & USER_IPS::operator=(const USER_IPS & i)
{
ips.clear();
copy(i.ips.begin(), i.ips.end(), back_inserter(ips));
return *this;
}
//-----------------------------------------------------------------------------
inline const IP_MASK & USER_IPS::operator[](int idx) const
{
return ips[idx];
}
//-----------------------------------------------------------------------------
inline string USER_IPS::GetIpStr() const
{
stringstream s;
unsigned int i = 0;

if (ips.size() == 0)
    {
    s << "";
    return s.str();
    }

if (ips[0].ip == 0)
    {
    s << "*";
    return s.str();
    }

while(1)
    {
    s << inet_ntostr(ips[i].ip);
    //ips[i].mask == 32 ? (cout << "") : (cout << "/" << ips[i].mask);
    if (i + 1 == ips.size()) break;
    s << ",";
    i++;
    }
return s.str();
}
//-----------------------------------------------------------------------------
inline int USER_IPS::Count() const
{
    return ips.size();
}
//-----------------------------------------------------------------------------
inline bool USER_IPS::IsIPInIPS(uint32_t ip) const
{
if (ips[0].ip == 0)
    return true;
for (unsigned int i = 0; i < ips.size(); i++)
    {
    if ((ip & CalcMask(ips[i].mask)) == ips[i].ip)
        return true;
    }
return false;
}
//-----------------------------------------------------------------------------
inline bool USER_IPS::OnlyOneIP() const
{
if (ips.size() == 1 && ips[0].mask == 32)
    return true;

return false;
}
//-----------------------------------------------------------------------------
inline uint32_t USER_IPS::CalcMask(unsigned int msk) const
{
if (msk > 32) return 0;
return htonl(0xFFffFFff << (32 - msk));
}
//-----------------------------------------------------------------------------
inline ostream & operator<< (ostream & o, const USER_IPS & i)
{
return o << i.GetIpStr();
}
//-----------------------------------------------------------------------------
inline stringstream & operator<< (stringstream & s, const USER_IPS & i)
{
s << i.GetIpStr();
return s;
}
//-----------------------------------------------------------------------------
inline const USER_IPS StrToIPS(const string & ipsStr) throw(string)
{
USER_IPS ips;
char * paddr;
IP_MASK im;
vector<string> ipMask;
string err;
if (ipsStr.empty())
    {
    err = "Incorrect IP address.";
    throw(err);
    }

if (ipsStr[0] == '*' && ipsStr.size() == 1)
    {
    im.ip = 0;
    im.mask = 0;
    ips.ips.push_back(im);
    return ips;
    }

char * str = new char[ipsStr.size() + 1];
strcpy(str, ipsStr.c_str());
char * pstr = str;
while ((paddr = strtok(pstr, ",")))
    {
    pstr = NULL;
    ipMask.push_back(paddr);
    }

delete[] str;

for (unsigned int i = 0; i < ipMask.size(); i++)
    {
    char str[128];
    char * strIp;
    char * strMask;
    strcpy(str, ipMask[i].c_str());
    strIp = strtok(str, "/");
    if (strIp == NULL)
        {
        err = "Incorrect IP address " + ipsStr;
        throw(err);
        }
    strMask = strtok(NULL, "/");

    im.ip = inet_addr(strIp);
    if (im.ip == INADDR_NONE)
        {
        err = "Incorrect IP address: " + string(strIp);
        throw(err);
        }

    im.mask = 32;
    if (strMask != NULL)
        {
        int m;
        if (str2x(strMask, m) != 0)
            {
            err = "Incorrect mask: " + string(strMask);
            throw(err);
            }
        im.mask = m;

        if (im.mask < 0 || im.mask > 32)
            {
            err = "Incorrect mask: " + string(strMask);
            throw(err);
            }

        if ((im.ip & ips.CalcMask(im.mask)) != im.ip)
            {
            err = "Address does'n match mask: " + string(strIp) + "/" + string(strMask);
            throw(err);
            }
        }
    ips.ips.push_back(im);
    }

return ips;
}
//-------------------------------------------------------------------------
#endif //USER_IPS_H


