#ifndef USER_TRAFF_H
#define USER_TRAFF_H
#include <iostream>
#include <sstream>
#include <inttypes.h>
#include <string.h>

#include "stg_const.h"

using namespace std;

enum TRAFF_DIRECTION {TRAFF_UPLOAD, TRAFF_DOWNLOAD};

class DIR_TRAFF
{   
    friend ostream & operator<< (ostream & o, const DIR_TRAFF & traff);
    friend stringstream & operator<< (stringstream & s, const DIR_TRAFF & traff);

public:
    //-------------------------------------------------------------------------
    DIR_TRAFF();
    DIR_TRAFF(const DIR_TRAFF & ts);
    DIR_TRAFF(const uint64_t * tr);
    DIR_TRAFF operator=(const DIR_TRAFF & ts);
    uint64_t operator[](int idx) const;
    uint64_t & operator[](int idx);
    DIR_TRAFF operator+(const DIR_TRAFF & ts);
    void TraffStatToArray(uint64_t * tr);
    
private:
    uint64_t    traff[DIR_NUM];
};
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline DIR_TRAFF::DIR_TRAFF()
{
memset(traff, 0, sizeof(uint64_t) * DIR_NUM);
}
//-----------------------------------------------------------------------------
inline DIR_TRAFF::DIR_TRAFF(const DIR_TRAFF & ts)
{
memcpy(traff, ts.traff, sizeof(uint64_t) * DIR_NUM);
}
//-----------------------------------------------------------------------------
inline DIR_TRAFF::DIR_TRAFF(const uint64_t * tr)
{
memcpy(traff, tr, sizeof(uint64_t) * DIR_NUM);
}
//-----------------------------------------------------------------------------
inline DIR_TRAFF DIR_TRAFF::operator=(const DIR_TRAFF & ts)
{
if (&ts == this)
    return *this;
    
memcpy(traff, ts.traff, sizeof(uint64_t) * DIR_NUM);
return *this;
};
//-----------------------------------------------------------------------------
inline uint64_t & DIR_TRAFF::operator[](int idx)
{
return traff[idx];
};
//-----------------------------------------------------------------------------
inline uint64_t DIR_TRAFF::operator[](int idx) const
{
return traff[idx];
};
//-----------------------------------------------------------------------------
inline DIR_TRAFF DIR_TRAFF::operator+(const DIR_TRAFF & ts)
{
for (int i = 0; i < DIR_NUM; i++)
    {
    traff[i] = traff[i] + ts.traff[i];
    }
return *this;
};
//-----------------------------------------------------------------------------
inline void DIR_TRAFF::TraffStatToArray(uint64_t * tr)
{
for (int i = 0; i < DIR_NUM; i++) 
    tr[i] = traff[i];
}
//-----------------------------------------------------------------------------
inline ostream & operator<< (ostream & o, const DIR_TRAFF & traff)
{
return o 
    << traff[0] << ","
    << traff[1] << ","
    << traff[2] << ","
    << traff[3] << ","
    << traff[4] << ","
    << traff[5] << ","
    << traff[6] << ","
    << traff[7] << ","
    << traff[8] << ","
    << traff[9];
}
//-----------------------------------------------------------------------------
inline stringstream & operator<< (stringstream & s, const DIR_TRAFF & traff)
{
s<< traff[0] << ","
 << traff[1] << ","
 << traff[2] << ","
 << traff[3] << ","
 << traff[4] << ","
 << traff[5] << ","
 << traff[6] << ","
 << traff[7] << ","
 << traff[8] << ","
 << traff[9];
return s;
}
//-----------------------------------------------------------------------------

#endif


