
//---------------------------------------------------------------------------
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "smtp.h"

//-----------------------------------------------------------------------------
bool SMTP::IsError(int res)
{
#ifdef WINDOWS
return res == SOCKET_ERROR;
#else
return res < 0;
#endif
}
//-----------------------------------------------------------------------------
SMTP::SMTP()
{
#ifdef WINDOWS
int res;
res = WSAStartup(MAKEWORD(1,1), &wsaData);

if (res)
    return;
#endif

port = 25;

SMTPSocket = socket(PF_INET, SOCK_STREAM, 0);

#ifdef WINDOWS
if (SMTPSocket == INVALID_SOCKET)
    {
    WSACleanup();
    // TODO error code
    return;
    }
strcpy(charset, "Windows-1251");
#else
if (SMTPSocket < 0)
    {
    return;
    }
strcpy(charset, "Koi8-u");
#endif
}
//-----------------------------------------------------------------------------
SMTP::~SMTP()
{
#ifdef WINDOWS
closesocket(SMTPSocket);
WSACleanup();
#else
close(SMTPSocket);
#endif
}
//-----------------------------------------------------------------------------
//   SMTP 
int SMTP::Connect()
{
int res;
unsigned long ip;
struct hostent he;

ip = inet_addr(host);

if (ip == INADDR_NONE)
    {
    struct hostent * hetmp = gethostbyname(host);
    if (hetmp == NULL)
        {
        strcpy(error, "Server cannot be connected");
        #ifdef WINDOWS
        closesocket(SMTPSocket);
        #else
        close(SMTPSocket);
        #endif
        return false;
        }
    memcpy(&he, hetmp, sizeof(he));
    ip = *((long*)he.h_addr_list[0]);
    }
outerAddr.sin_family = AF_INET;
outerAddr.sin_addr.s_addr = ip;
outerAddr.sin_port = htons(port);

res = connect(SMTPSocket, (struct sockaddr*)&outerAddr, sizeof(outerAddr));

if (IsError(res))
    {
    strcpy(error, "Server cannot be connected");
    #ifdef WINDOWS
    closesocket(SMTPSocket);
    #else
    close(SMTPSocket);
    #endif
    return false;
    }
else
    {
    if(!CheckResponse(CONNECTION_CHECK))
        return false;

    char buf [512];
    
    char * f;
    f = new char[strlen(from)+1];
    memset(f, 0, strlen(from)+1);
    int i = 0;
    while(from[i] && from[i] != '@')
        f[i] = from[i++];

    if (f[0] == 0)
        strcpy(f, "fake");

    sprintf(buf, "HELO %s\r\n", f);
    //printf("%s\n", buf);
    delete[] f;

    res = send(SMTPSocket, buf, strlen(buf), 0);

    if (!CheckResponse(HELLO_CHECK))
        return false;

    return true;
    }
}
//-----------------------------------------------------------------------------
//   
void SMTP::SetHost(const char * h)
{
strncpy(host, h, STR_LEN);
}
//-----------------------------------------------------------------------------
//   
char * SMTP::GetHost()
{
return host;
}
//-----------------------------------------------------------------------------
void SMTP::SetPort(unsigned short p)
{
port = p;
}
//-----------------------------------------------------------------------------
unsigned short SMTP::GetPort()
{
return port;
}
//-----------------------------------------------------------------------------
void SMTP::SetSubject(const char * subj)
{
strncpy(subject, subj, STR_LEN);
}
//-----------------------------------------------------------------------------
char * SMTP::GetSubject()
{
return subject;
}
//-----------------------------------------------------------------------------
int SMTP::Disconnect()
{
//  SMTP   "QUIT"
int res;
char buf[256];

sprintf (buf, "QUIT \r\n");

res = send(SMTPSocket, buf, strlen(buf), 0);

if (res < 0)
    return false;

if (CheckResponse(QUIT_CHECK) == false)
    {
    #ifdef WINDOWS
    closesocket(SMTPSocket);
    #else
    close(SMTPSocket);
    #endif
    return false;
    }
else
    {
    #ifdef WINDOWS
    closesocket(SMTPSocket);
    #else
    close(SMTPSocket);
    #endif
    return true;
    }
}
//-----------------------------------------------------------------------------
int SMTP::Mail()
{
//  SMTP   "MAIL"
int res;
char buf[256];

sprintf (buf, "MAIL From:<%s>\r\n", from);

res = send(SMTPSocket, buf, strlen(buf), 0);

if (res < 0)
    return false;

if (!CheckResponse(MAIL_CHECK))
    return false;

return true;
}
//-----------------------------------------------------------------------------
void SMTP::SetFrom(const char * f)
{
strncpy(from, f, STR_LEN);
}
//-----------------------------------------------------------------------------
char * SMTP::GetFrom()
{
return from;
}
//-----------------------------------------------------------------------------
void SMTP::SetTo(const char * t)
{
strcpy(to, t);
}
//-----------------------------------------------------------------------------
int SMTP::SendTo()
{
char buf[300];
int res;

sprintf (buf, "RCPT TO:<%s>\r\n", to);

res = send(SMTPSocket, buf, strlen(buf), 0);

if (res < 0)
    return false;

if (!CheckResponse(RCPT_CHECK))
    return false;

return true;
}
//-----------------------------------------------------------------------------
char * SMTP::GetTo()
{
//   TO
return to;
}
//-----------------------------------------------------------------------------
int SMTP::Data()
{
//  SMTP   "DATA"
char buf[256];
int res;

sprintf (buf, "DATA\r\n");

res = send(SMTPSocket, buf, strlen(buf), 0);
if (res <=0 )
    return false;

char wd[][4]={
{"Sun"},
{"Mon"},
{"Tue"},
{"Wed"},
{"Thu"},
{"Fri"},
{"Sat"}};

char months[][4]={
{"Jan"},
{"Feb"},
{"Mar"},
{"Apr"},
{"May"},
{"Jun"},
{"Jul"},
{"Aug"},
{"Sep"},
{"Oct"},
{"Nov"},
{"Dec"}};


struct tm * t;
time_t tt = time(NULL);
t = localtime(&tt);
char timeStr[10];

sprintf(timeStr, "%s%d:%s%d:%s%d",
    t->tm_hour < 10 ? "0" : "", t->tm_hour,
    t->tm_min < 10 ? "0" : "",  t->tm_min,
    t->tm_sec < 10 ? "0" : "",  t->tm_sec);

char timeZoneStr[10];
int tz;
#ifdef WINDOWS
    tz = _timezone;
#else
    #ifdef LINUX
    tz = timezone;
    #else
    time_t tm = time(NULL);
    tz = mktime(localtime(&tm)) - mktime(gmtime(&tm));
    #endif
#endif

int h = tz / 3600;
int m = (abs(tz) / 60) % 60;
sprintf(timeZoneStr, "%s%s%d%s%d",
    h >= 0 ? "-" : "+",
    abs(h) < 10 ? "0" : "",
    abs(h),
    m < 10 ? "0" : "",
    m);


sprintf (buf, "Date:%s, %d %s %d %s %s\r\n", wd[t->tm_wday], t->tm_mday, months[t->tm_mon], t->tm_year+1900, timeStr, timeZoneStr);
res = send(SMTPSocket, buf, strlen(buf), 0);
if (res <= 0)
    return false;

sprintf (buf, "From:%s\r\n", from);
res = send(SMTPSocket, buf, strlen(buf), 0);
if (res <= 0)
    return false;


sprintf (buf, "To:%s\r\n", to);
res = send(SMTPSocket, buf, strlen(buf), 0);
if (res <= 0)
    return false;

sprintf (buf, "Subject:%s\r\n", subject);
res = send(SMTPSocket, buf, strlen(buf), 0);
if (res <= 0)
    return false;

sprintf (buf, "Content-Type: text/plain; charset=%s\r\n", charset);
res = send(SMTPSocket, buf, strlen(buf), 0);
if (res <= 0)
    return false;

sprintf (buf, "\r\n");
res = send(SMTPSocket, buf, strlen(buf), 0);

if (res <= 0)
    return false;

if (!CheckResponse(DATA_CHECK))
    return false;
return true;
}
//-----------------------------------------------------------------------------
int SMTP::Send(const char * body)
{
int res;
char buf[256];

sprintf (buf, "%s\r\n", body);
res = send(SMTPSocket, buf, strlen(buf), 0);
return !IsError(res);
}
//-----------------------------------------------------------------------------
int SMTP::SendDot()
{
int res;
char buf[256];

sprintf (buf, "\r\n.\r\n");
res = send(SMTPSocket, buf, strlen(buf), 0);
if (IsError(res))
    return false;

if (!CheckResponse(DATA_END_CHECK))
    return false;

return true;
}
//-----------------------------------------------------------------------------
int SMTP::CheckResponse(int Type)
{
//      
char buf[255];
char temp[4];
int res;

memset(buf, 0, sizeof(buf));
memset(temp, 0, sizeof(temp));

//    
res = recv(SMTPSocket, buf, sizeof(buf), 0);
if (res <= 0)
    return false;

strncpy(temp, buf, 3);
int temp2 = atoi(temp);
switch (Type)
    {
    case CONNECTION_CHECK:
        if (temp2 != 220)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;

    case HELLO_CHECK:
        if (temp2 != 250)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;

    case MAIL_CHECK:
        if (temp2 != 250)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;

    case RCPT_CHECK:
        if (temp2 != 250)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;

    case DATA_START_CHECK:
        if (temp2 != 354)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;

    case DATA_END_CHECK:
        if (temp2 != 250)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;

    case QUIT_CHECK:
        if (temp2 != 221)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;

    case DATA_CHECK:
        if (temp2 != 354)
            {
            strncpy(error, GetError(buf), STR_LEN);
            return false;
            }
        break;
    }
return true;
}
//-----------------------------------------------------------------------------
//    
char * SMTP::GetErrorMessage()
{
return error;
}
//-----------------------------------------------------------------------------
//     -  
char * SMTP::GetError(char * response)
{
error[0] = 0;

if(strstr(response, "211"))
    strcpy(error, "System status or system help reply");
if(strstr(response, "214"))
    strcpy(error, "Help Message");
if(strstr(response, "220"))
    strcpy(error, "Service is ready");
if(strstr(response, "221"))
    strcpy(error, "Service closing transmission channel");
if(strstr(response, "250"))
    strcpy(error, "Requested mail action okay, completed");
if(strstr(response, "251"))
    strcpy(error, "user not local: will forward to forward path");
if(strstr(response, "354"))
    strcpy(error, "Start mail input; end with <CRLF>.<CRLF>");
if (error[0] == 0)
    sprintf(error, "No Error Number is matched with %s", response);

return error;
}
//-----------------------------------------------------------------------------


