#include <ctpp/ctpp.hpp>
#include <iostream>
#include <unistd.h>

#include "common.h"
#include "htmler.h"
#include "body_tmpl.h"

using namespace std;

//-----------------------------------------------------------------------------
HTMLER::HTMLER()
{
    fileName = "ping.html";
    htmlUpdatePeriod = 5;

    char * pingsv = ""
    #ifndef DISABLE_PINGER_V1
    "PING1, "
    #endif
    #ifndef DISABLE_PINGER_V2
    "PING2"
    #endif
    ;

    char * os = ""
    #ifdef LINUX
    "Linux"
    #endif
    #ifdef FREE_BSD
    "FreeBSD"
    #endif
    #ifdef FREE_BSD5
    "FreeBSD"
    #endif
    #ifdef WIN32
    "Windows"
    #endif
    ;

    sprintf(ver, "PingSTAT v2.01 /%s /%s", pingsv, os);
}
//-----------------------------------------------------------------------------
void HTMLER::SetFileName(const string & fileName)
{
HTMLER::fileName = fileName;
}
//-----------------------------------------------------------------------------
void HTMLER::SetTplFileName(const string & fileName)
{
tplFileName = fileName;
}
//-----------------------------------------------------------------------------
void HTMLER::SetHTMLUpdatePeriod(time_t period)
{
htmlUpdatePeriod = period;
}
//-----------------------------------------------------------------------------
void HTMLER::SetHTMLEncoding(const string & encoding)
{
HTMLEncoding = encoding;
}
//-----------------------------------------------------------------------------
int HTMLER::WriteHtml(const vector<PINGER> & pinger) const
{
using namespace template_parser_ns;

param_data * pData = new param_data(param_data::HASH);
string s;
pData->hash_insert_new_var("Html_encoding", HTMLEncoding);
pData->hash_insert_new_var("update_period", x2str(htmlUpdatePeriod, s));
pData->hash_insert_new_var("pingers_num", x2str(pinger.size(), s));
pData->hash_insert_new_var("ProgVersion", ver);

param_data * pArray = pData->hash_insert_new_array("PING_INFO");

param_data * pHash;

int a = 0;

vector<PINGER>::const_iterator pingIter;
pingIter = pinger.begin();
while(pingIter != pinger.end())
    {
    a++;
    pHash = pArray->array_insert_new_hash();
    pHash->hash_insert_new_var("Version", pingIter->GetVersion() ? x2str(pingIter->GetVersion(), s) : "-");
    pHash->hash_insert_new_var("IP", inet_ntostr(pingIter->GetIP()));
    pHash->hash_insert_new_var("PingerStatus", pingIter->GetStatusName());
    pHash->hash_insert_new_var("Name", pingIter->GetName());
    pHash->hash_insert_new_var("Address", pingIter->GetAddress());
    pHash->hash_insert_new_var("Descr", pingIter->GetDescr());

    if (pingIter->IsPacketRecvd())
        {
        pHash->hash_insert_new_var("LastUpdateTime", LogDate(pingIter->GetLastUpdateTime()));
        }
    else
        {
        pHash->hash_insert_new_var("LastUpdateTime", "---");
        }

    char sss[30];

    if (pingIter->GetStatus() != PINGER::St_Dead)
        {
        sprintf(sss, "%.2f", pingIter->GetAvrgUpdateTime());
        pHash->hash_insert_new_var("AvrgUpdateTime", sss);
        }
    else
        {
        pHash->hash_insert_new_var("AvrgUpdateTime", "---");
        }

    for (int j = 0; j < 8; j++)
        {
        char bitNum[15];
        char bitState[50];

        sprintf(bitNum, "Bit%d", j);
        sprintf(bitState, "Bit%dState", j);

        if (pingIter->GetBit(j).GetDisabled())
            {
            pHash->hash_insert_new_var(bitNum, "---");
            pHash->hash_insert_new_var(bitState, "_disabled");
            }
        else
            {
            string name;
            if (pingIter->GetBit(j).GetName() != "")
                {
                name = pingIter->GetBit(j).GetName() + " ";
                }
            pHash->hash_insert_new_var(bitNum, name + pingIter->GetBit(j).GetStateName());
            pHash->hash_insert_new_var(bitState, x2str(pingIter->GetBit(j).GetState(), s));
            }
        }

    char tempStr[10];

    if (pingIter->GetVersion() == 2 && !pingIter->GetTempDisabled())
        sprintf(tempStr, "%.1f", pingIter->GetTemp());
    else
        {

        sprintf(tempStr, "---");
        }

    pHash->hash_insert_new_var("Temperature", tempStr);

    if (pingIter->GetTempDisabled() || pingIter->GetVersion() == 1)
        pHash->hash_insert_new_var("TemperatureState", "temperature_disabled");
    else
        pHash->hash_insert_new_var("TemperatureState", pingIter->IsTempStatusOk() ? "TemperatureStateOk" : "TemperatureStateError");

    ++a;
    ++pingIter;
    }

pHash->hash_insert_new_var("global_param", "Replaced Parameter");

template_text oTemplate;

loader_base * pLoaderBase = new loader_base();

v_include_dir vIncludeDir;
vIncludeDir.push_back("./");
oTemplate.set_include_dir(vIncludeDir);
string tplFileName1 = tplFileName + ".tpl";
ReadTpl();
WriteTpl();

pLoaderBase->load_file(tplFileName1);
oTemplate.parse(pLoaderBase -> get_data());
oTemplate.param(pData);
string sOutput;
bool bDummy = true;
sOutput.assign(oTemplate.output(bDummy));

delete pData;
delete pLoaderBase;

FILE * f = fopen(fileName.c_str(), "wt");
if (!f)
    {
    strError = "Cannot open file \'" + fileName + "\' for writing";
    return -1;
    }

fprintf(f, "%s", sOutput.c_str());

fclose(f);
RemoveTpl();
return 0;
}
//-----------------------------------------------------------------------------
string HTMLER::GetStrError() const
{
return strError;
}
//-----------------------------------------------------------------------------
int HTMLER::WriteTpl() const
{
string fileName = tplFileName + ".tpl";
FILE * f = fopen(fileName.c_str(), "wb");
if (!f)
    {
    strError = "Cannot open file \'" + fileName + "\' for writing";
    return -1;
    }

fwrite(tplBuff, strlen(tplBuff), 1, f);
fwrite(body_tpl, strlen(body_tpl), 1, f);

fclose(f);
return 0;
}
//--------------------------------------------------------------------------------
int HTMLER::ReadTpl() const
{
int len = -1;
FILE * f = fopen(tplFileName.c_str(), "rb");
if (!f)
    {
    strError = "Cannot open file \'" + tplFileName + "\' for writing";
    return len;
    }

len = fread(tplBuff, sizeof(tplBuff), 1, f);

fclose(f);

return len;
}
//--------------------------------------------------------------------------------
void HTMLER::RemoveTpl() const
{
string fileName = tplFileName + ".tpl";
unlink(fileName.c_str());
}
//--------------------------------------------------------------------------------

