#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>
#include <stdarg.h>
#include <netdb.h>
#include <sys/wait.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <arpa/inet.h>
#include <algorithm>

#include "main.h"
#include "readconf.h"
#include "htmler.h"
#include "pinger.h"
#include "packet_parser.h"

using namespace std;

//--------------------------------------------------------------------------------
#define alarmTime   1
#define timeOut     30
//#define NODAEMON
#define port 4000
#define gettext(a) (a)

static int nonstop = 1;

HTMLER htmler;
vector<PINGER> pingers;
//--------------------------------------------------------------------------------
int PrepareNet(int * sock, struct sockaddr_in * msin)
{
if ((*sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
    {
    fprintf(stderr, gettext("Create socket error: %s"), strerror(errno));
    return -1;
    }

memset(msin, 0, sizeof(*msin));

msin->sin_family = AF_INET;
msin->sin_addr.s_addr = htonl(INADDR_ANY);
msin->sin_port = htons(port);

/*int lng = 1;

if (0 != setsockopt(*sock, SOL_SOCKET, SO_REUSEADDR, &lng, 4))
    {
    fprintf(stderr, "Setsockopt error: %s\n", strerror(errno));
    return -1;
    }*/

if (bind(*sock, (struct sockaddr*)msin, sizeof(*msin)) < 0)
    {
    fprintf(stderr, "Bind error: %s\n", strerror(errno));
    return -1;
    }

return 0;
}
//--------------------------------------------------------------------------------
int RecvData(int * sock, struct sockaddr_in * msin, const PACKET_PARSER & packetParser)
{
char data[2048];
while (nonstop)
    {
    socklen_t msinlen = sizeof(*msin);
    int len = recvfrom(*sock, data, sizeof(data), 0, (struct sockaddr*)msin, &msinlen);
    //printf("recvfrom %s len=%d\n", inet_ntostr(*(uint32_t*)&msin->sin_addr), len);
    packetParser.Parse(*(uint32_t*)&msin->sin_addr, data, len);
    }

return 0;
}
//--------------------------------------------------------------------------------
void CatchAlarm(int sig_num)
{
static int a = 0;
a++;
if (a % 5)
    {
    htmler.WriteHtml(pingers);
    }

for_each(pingers.begin(), pingers.end(), mem_fun_ref(&PINGER::Update));

alarm(alarmTime);
}
//--------------------------------------------------------------------------------
void CatchTerm(int sig_num)
{
nonstop = 0;
exit(0);
}
//--------------------------------------------------------------------------------
int WriteLog(char * line)
{
FILE * f;
char timeStr[100];
time_t tm = time(NULL);

strncpy(timeStr, ctime(&tm), 99);

f = fopen("/var/log/pingstat.log", "at");
if (f)
    {
    fprintf(f, strtok(timeStr, "\n"));
    fprintf(f, " - ");
    fprintf(f, line);
    fprintf(f, "\n");
    fclose(f);
    }
return 0;
}
//-----------------------------------------------------------------------------
bool CheckFileWritable(const string fileName)
{
FILE * f;
f = fopen(fileName.c_str(), "wb");
if (!f)
    return false;

fclose(f);
return true;
}
//-----------------------------------------------------------------------------
int main(int argc, char * argv[])
{
int sock;
struct sockaddr_in msin = {AF_INET};
string configFileName = "./pingstat.conf";

if (argc == 2)
    {
    configFileName = argv[1];
    }
else if (argc > 2)
    {
    printf("Too many parameters\n");
    return -1;
    }

SETTINGS settings(configFileName);

if (settings.ReadSettings() < 0)
{
    printf("ReadSettings error: %s\n", settings.GetStrError().c_str());
    return -1;
}

//settings.Print();

PACKET_PARSER packetParser(&pingers);

for (int i = 0; i < settings.GetPingerCount(); i++)
    {
    PingerParams pingerParams;
    settings.GetPingerParams(i, &pingerParams);
    //pingerParams.
    PINGER pinger(pingerParams);
    pingers.push_back(pinger);
    }

PINGER::SetScriptName(settings.GetScriptName());
PINGER::SetTimeouts(settings.GetPingWarningTimeout(), settings.GetPingTimeout());

if (PrepareNet(&sock, &msin) < 0)
    exit(-1);

signal(SIGALRM, CatchAlarm);
signal(SIGTERM, CatchTerm);
signal(SIGINT, CatchTerm);

if (!CheckFileWritable(settings.GetOutputFileName()))
    {
    printf("Cannot write file \'%s\'. Check permissions\n", settings.GetOutputFileName().c_str());
    return -1;
    }

#ifndef NODAEMON
switch (fork())
    {
    case -1:
        printf("Fork error!\n");
        exit(1);
        break;
    case 0:
        close(0);
        close(1);
        close(2);
        setsid();
        break;
    default:
        exit(0);
        break;
    }
#endif

htmler.SetHTMLUpdatePeriod(settings.GetUpdateWebPagePeriod());
htmler.SetHTMLEncoding(settings.GetHTMLEncoding());
htmler.SetFileName(settings.GetOutputFileName());
htmler.SetTplFileName(settings.GetTplFileName());

alarm(alarmTime);

htmler.WriteHtml(pingers);

RecvData(&sock, &msin, packetParser);

return 0;
}
//--------------------------------------------------------------------------------


