#include "pinger.h"

using namespace std;

std::string     PINGER::scriptName;
time_t   PINGER::warningTimeout;
time_t   PINGER::errorTimeout;

//-----------------------------------------------------------------------------
PINGER_BIT::PINGER_BIT(bool normalValue)
:normalValue(normalValue)
{
trueStateName = "1";
falseStateName = "0";
name = "";
value = 0;
disabled = false;
}
//-----------------------------------------------------------------------------
PINGER_BIT::~PINGER_BIT()
{

}
//-----------------------------------------------------------------------------
bool PINGER_BIT::GetDisabled() const
{
return disabled;
}
//-----------------------------------------------------------------------------
void PINGER_BIT::SetDisabled(bool d)
{
disabled = d;
}
//-----------------------------------------------------------------------------
void PINGER_BIT::SetValue(bool val)
{
value = val;
}
//-----------------------------------------------------------------------------
bool PINGER_BIT::GetValue() const
{
return value;
}
//-----------------------------------------------------------------------------
string PINGER_BIT::GetName() const
{
return name;
}
//-----------------------------------------------------------------------------
void PINGER_BIT::SetName(string name)
{
PINGER_BIT::name = name;
}
//-----------------------------------------------------------------------------
/*bool PINGER_BIT::GetDisabled() const
{
return disabled;
}
//-----------------------------------------------------------------------------
void PINGER_BIT::SetDisabled(bool d)
{
disabled = d;
}*/
//-----------------------------------------------------------------------------
void PINGER_BIT::SetStateNames(string name0, string name1)
{
trueStateName = name1;
falseStateName = name0;
}
//-----------------------------------------------------------------------------
bool PINGER_BIT::GetState() const
{
return normalValue == value;
}
//-----------------------------------------------------------------------------
string PINGER_BIT::GetStateName() const
{
return (normalValue == value) ? trueStateName : falseStateName;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
PINGER::PINGER(const PingerParams & pingerParams)
{
ip = pingerParams.ip;
address = pingerParams.address;
description = pingerParams.description;
k = pingerParams.temp.k;
b = pingerParams.temp.b;
maxTemp = pingerParams.temp.maxTemp;
minTemp = pingerParams.temp.minTemp;
tempDisabled = pingerParams.temp.disabled;
name = pingerParams.name;

isPacketRecvd = false;

//temp = 36.6;
acp = 170;

version = 0;

warningTimeout = 30;
errorTimeout = 60;

status = St_Live;

for (unsigned i = 0; i < 8; i++)
    {
    bitOrder[i] = pingerParams.bitPapams[i].order;
    }

int nv;

for (int i = 0; i < 8; i++)
    {
    nv = pingerParams.bitPapams[i].normalValue;
    PINGER_BIT b(nv);
    b.SetStateNames(pingerParams.bitPapams[i].name_0, pingerParams.bitPapams[i].name_1);
    b.SetName(pingerParams.bitPapams[i].name);
    b.SetDisabled(pingerParams.bitPapams[i].disabled);
    byte.push_back(b);
    }

packetsTime.push_front(time(NULL));
tempStatus = true;
}
//-----------------------------------------------------------------------------
PINGER::~PINGER()
{

}
//-----------------------------------------------------------------------------
void PINGER::SetIP(uint32_t ip)
{
PINGER::ip = ip;
}
//-----------------------------------------------------------------------------
void PINGER::SetAddress(const string & addr)
{
address = addr;
}
//-----------------------------------------------------------------------------
void PINGER::SetDescr(const string & descr)
{
description = descr;
}
//-----------------------------------------------------------------------------
void PINGER::SetScriptName(const string & sn)
{
scriptName = sn + " ";
}
//-----------------------------------------------------------------------------
void PINGER::SetMaxTemp(double t)
{
maxTemp = t;
}
//-----------------------------------------------------------------------------
void PINGER::SetMinTemp(double t)
{
minTemp = t;
}
//-----------------------------------------------------------------------------
void PINGER::SetKB(double k, double b)
{
PINGER::k = k;
PINGER::b = b;
}
//-----------------------------------------------------------------------------
void PINGER::SetTimeouts(time_t warningTimeout, time_t errorTimeout)
{
PINGER::warningTimeout = warningTimeout;
PINGER::errorTimeout = errorTimeout;
}
//-----------------------------------------------------------------------------
void PINGER::Update(char bb, int acp, int ver) const
{
isPacketRecvd = true;
version = ver;
double told = PINGER::acp * k + b;
double tnew = acp * k + b;

if (status != St_Live)
    {
    Alive();
    }

PrepareScriptParams();
if (!packetsTime.empty() && !tempDisabled)
    {
    //printf("Check temp told=%.2f tnew=%.2f\n", told, tnew);
    //printf("minTemp %.2f maxTemp=%.2f\n", minTemp, maxTemp);
    if (told > minTemp && tnew <= minTemp)
        {
        tempStatus = false;
        changedValue = "MinTempError";
        string command = scriptName + changedValue + scriptParams;
        system(command.c_str());
        //printf("Check temp %s\n", changedValue.c_str());
        }
    else if (told < minTemp && tnew >= minTemp)
        {
        tempStatus = true;
        changedValue = "MinTempOk";
        string command = scriptName + changedValue + scriptParams;
        system(command.c_str());
        //printf("Check temp %s\n", changedValue.c_str());
        }
    else if (told < maxTemp && tnew >= maxTemp)
        {
        tempStatus = false;
        changedValue = "MaxTempError";
        string command = scriptName + changedValue + scriptParams;
        system(command.c_str());
        //printf("Check temp %s\n", changedValue.c_str());
        }
    else if (told > maxTemp && tnew <= maxTemp)
        {
        tempStatus = true;
        changedValue = "MaxTempOk";
        string command = scriptName + changedValue + scriptParams;
        system(command.c_str());
        //printf("Check temp %s\n", changedValue.c_str());
        }
    else if (told < minTemp && tnew > maxTemp)
        {
        tempStatus = false;
        changedValue = "MaxTempError";
        string command = scriptName + changedValue + scriptParams;
        system(command.c_str());
        //printf("Check temp %s\n", changedValue.c_str());
        }
    else if (told > maxTemp && tnew < minTemp)
        {
        tempStatus = false;
        changedValue = "MinTempError";
        string command = scriptName + changedValue + scriptParams;
        system(command.c_str());
        //printf("Check temp %s\n", changedValue.c_str());
        }
    }

for (int i = 0; i < 8; i++)
    {
    if (packetsTime.empty())
        {
        byte[i].SetValue((bb & (1 << i)) ? 1 : 0);
        }
    else if (byte[i].GetValue() != (bb & (1 << i) ? 1 : 0) && !byte[i].GetDisabled())
        {
        byte[i].SetValue((bb & (1 << i)) ? 1 : 0);
        PrepareScriptParams();
        changedValue = string("Bit") + char('0' + bitOrder[i]);
        string command = scriptName + changedValue + scriptParams;
        system(command.c_str());
        }
    }

PINGER::acp = acp;
/*if (!packetsTime.empty() && *packetsTime.begin() == 0)
    *packetsTime.begin() = time(NULL);
else*/
    packetsTime.push_front(time(NULL));

if (packetsTime.size() > 10)
    {
    packetsTime.pop_back();
    }
}
//-----------------------------------------------------------------------------
void PINGER::Update() const
{
if (packetsTime.empty())
    return;

if (time(NULL) - *packetsTime.begin() > errorTimeout && status < St_Dead)
    {
    Dead();
    }
else if (time(NULL) - *packetsTime.begin() > warningTimeout && status < St_Warning)
    {
    Warning();
    }
}
//-----------------------------------------------------------------------------
double PINGER::GetTemp() const
{
//printf("%.2f\n", acp * k + b);
return acp * k + b;
}
//-----------------------------------------------------------------------------
uint32_t PINGER::GetIP() const
{
return ip;
}
//-----------------------------------------------------------------------------
string PINGER::GetAddress() const
{
return address;
}
//-----------------------------------------------------------------------------
string PINGER::GetName() const
{
return name;
}
//-----------------------------------------------------------------------------
string PINGER::GetDescr() const
{
return description;
}
//-----------------------------------------------------------------------------
const PINGER_BIT & PINGER::GetBit(int num) const
{
if (num < 0 || num > 7)
    num = 0;

map<int, int>::const_iterator iter;
iter = bitOrder.find(num);

if (iter == bitOrder.end())
    num = 0;
else
    num = iter->second;

return byte[num];
}
//-----------------------------------------------------------------------------
time_t PINGER::GetLastUpdateTime() const
{
if (packetsTime.empty())
    {
    return 0;
    }
return *packetsTime.begin();
}
//-----------------------------------------------------------------------------
double PINGER::GetAvrgUpdateTime() const
{
if (packetsTime.size() < 2)
    {
    return 0.0;
    }

list<time_t>::const_iterator lastIter = packetsTime.end();
--lastIter;

return double(time(NULL) - *lastIter) / (packetsTime.size() - 1);
}
//-----------------------------------------------------------------------------
bool PINGER::operator<(const PINGER & rhs) const
{
return ip < rhs.ip;
}
//-----------------------------------------------------------------------------
bool PINGER::operator==(const PINGER & rhs) const
{
return ip == rhs.ip;
}
//-----------------------------------------------------------------------------
void PINGER::PrepareScriptParams() const
{
scriptParams = " ";
scriptParams += inet_ntostr(ip);
scriptParams += " \"";
//scriptParams += name;
//scriptParams += "\" \"";
scriptParams += address;
scriptParams += "\" \"";
scriptParams += description;
scriptParams += "\" ";

for (int i = 0; i < 8; i++)
    {
    scriptParams += byte[bitOrder[i]].GetState() ? "1" : "0";
    scriptParams += " \"";
    scriptParams += byte[bitOrder[i]].GetStateName();
    scriptParams += "\" ";
    }
scriptParams += " ";

//printf("params=%s\n", scriptParams.c_str());
}
//-----------------------------------------------------------------------------
void PINGER::Warning() const
{
PrepareScriptParams();
changedValue = "Warning";
string command = scriptName + changedValue + scriptParams;
system(command.c_str());
status = St_Warning;
}
//-----------------------------------------------------------------------------
void PINGER::Dead() const
{
changedValue = "Dead";
string command = scriptName + changedValue + scriptParams;
system(command.c_str());
status = St_Dead;
}
//-----------------------------------------------------------------------------
void PINGER::Alive() const
{
changedValue = "Alive";
string command = scriptName + changedValue + scriptParams;
system(command.c_str());
status = St_Live;
}
//-----------------------------------------------------------------------------
int PINGER::GetVersion() const
{
return version;
}
//-----------------------------------------------------------------------------
int PINGER::GetStatus() const
{
return status;
}
//-----------------------------------------------------------------------------
std::string PINGER::GetStatusName() const
{
switch (status)
    {
    case St_Live:       return "Live";
    case St_Warning:    return "Warning";
    default:            return "Dead";
    }
}
//-----------------------------------------------------------------------------
bool PINGER::IsTempStatusOk() const
{
return tempStatus;
}
//-----------------------------------------------------------------------------
bool PINGER::GetTempDisabled() const
{
return tempDisabled;
}
//-----------------------------------------------------------------------------
bool PINGER::IsPacketRecvd() const
{
return isPacketRecvd;
}
//-----------------------------------------------------------------------------
