#ifndef PINGER_H
#define PINGER_H

#ifdef LINUX
#include <stdint.h>
#endif

#ifdef FREE_BSD5
#include <inttypes.h>
#endif

#ifdef FREE_BSD
#include <sys/inttypes.h>
#endif


#include <string>
#include <list>
#include <vector>
#include <map>

#include "readconf.h"

//-----------------------------------------------------------------------------
class PINGER_BIT
{
public:
                        PINGER_BIT(bool normalValue);
    virtual             ~PINGER_BIT();

    virtual void        SetValue(bool val);
    virtual bool        GetValue() const;

    virtual std::string GetName() const;
    virtual void        SetName(std::string name);

    virtual bool        GetDisabled() const;
    virtual void        SetDisabled(bool disabled);

    virtual void        SetStateNames(std::string name0, std::string name1);

    virtual bool        GetState() const;
    virtual std::string GetStateName() const;

protected:
    bool        normalValue;
    bool        value;
    bool        disabled;
    std::string name;
    std::string trueStateName;
    std::string falseStateName;
};
//-----------------------------------------------------------------------------
class PINGER
{
public:
                        PINGER(const PingerParams & pingerParams);
                        ~PINGER();

    void                SetIP(uint32_t ip);
    void                SetAddress(const std::string & addr);
    void                SetDescr(const std::string & addr);
    static void         SetTimeouts(time_t warningTimeout, time_t errorTimeout);
    static void         SetScriptName(const std::string & scriptName);


    void                SetMaxTemp(double t);
    void                SetMinTemp(double t);
    void                SetKB(double k, double b);

    void                Update(char byte, int acp, int ver) const;
    void                Update() const;

    time_t              GetLastUpdateTime() const;
    double              GetAvrgUpdateTime() const;

    double              GetTemp() const;
    bool                GetTempDisabled() const;

    uint32_t            GetIP() const;
    std::string         GetAddress() const;
    std::string         GetName() const;
    std::string         GetDescr() const;

    bool                IsTempStatusOk() const;

    const PINGER_BIT  & GetBit(int num) const;

    bool                operator<(const PINGER & rhs) const;
    bool                operator==(const PINGER & rhs) const;

    int                 GetVersion() const;

    enum {St_Live, St_Warning, St_Dead};

    int                 GetStatus() const;
    std::string         GetStatusName() const;

    bool                IsPacketRecvd() const;

protected:

    void                PrepareScriptParams() const;
    void                Dead() const;
    void                Warning() const;
    void                Alive() const;

    uint32_t            ip;
    std::string         address;
    std::string         name;
    std::string         description;

    double              k;
    double              b;

    double              maxTemp;
    double              minTemp;
    bool                tempDisabled;
    mutable bool        tempStatus;
    //mutable double      temp;
    double              avrgTime;
    mutable int         acp;

    mutable bool        isPacketRecvd;


    mutable std::vector<PINGER_BIT> byte;
    mutable std::map<int, int>      bitOrder;

    mutable std::list<time_t>       packetsTime;

    static time_t                   warningTimeout;
    static time_t                   errorTimeout;
    mutable int                     status;
    mutable int                     version;

    static std::string              scriptName;
    mutable std::string             scriptParams;
    mutable std::string             changedValue;
};
//-----------------------------------------------------------------------------

#endif

