#include <iostream>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <set>
#include <algorithm>

#include "readconf.h"

using namespace std;

//-----------------------------------------------------------------------------
SETTINGS::SETTINGS(const string & configFileName)
{
SETTINGS::configFileName = configFileName;
}
//-----------------------------------------------------------------------------
SETTINGS::~SETTINGS()
{

}
//-----------------------------------------------------------------------------
int SETTINGS::ReadSettings()
{
const char * requiredOptions[] =
{
    "Script_name",
    //"Log_name",
    "Tpl_file_name",
    "Output_file_name",
    "Ping_warning_timeout",
    "Ping_timeout",
    "Update_web_page_period",
    "Html_encoding",
    NULL
};

DOTCONFDocument conf(DOTCONFDocument::CASEINSENSETIVE);
conf.setRequiredOptionNames(requiredOptions);

string ipAddr;

if (conf.setContent(configFileName.c_str()) == -1)
    {
    strError = "Cannot read file \'" + configFileName + "\'";
    return -1;
    }

const DOTCONFDocumentNode * node = conf.getFirstNode();

while (node)
    {
    if (strcasecmp(node->getName(), "Script_name") == 0)
        {
        if (node->getValue(0))
            {
            scriptName = node->getValue(0);
            }
        else
            {
            strError = "\'Script_name\' parameter value missed";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Log_name") == 0)
        {
        if (node->getValue(0))
            {
            logName = node->getValue(0);
            }
        else
            {
            strError = "\'Log_name\' parameter value missed";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Tpl_file_name") == 0)
        {
        if (node->getValue(0))
            {
            tplFileName = node->getValue(0);
            }
        else
            {
            strError = "\'Output_file_name\' parameter value missed";
            return -1;
            }
        }

        if (strcasecmp(node->getName(), "Output_file_name") == 0)
        {
        if (node->getValue(0))
            {
            outputFileName = node->getValue(0);
            }
        else
            {
            strError = "\'Tpl_file_name\' parameter value missed";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Html_encoding") == 0)
        {
        if (node->getValue(0))
            {
            HTMLEncoding = node->getValue(0);
            }
        else
            {
            strError = "\'Html_encoding\' parameter value missed";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Ping_warning_timeout") == 0)
        {
        if (node->getValue(0))
            {
            if (ParseIntInRange(node->getValue(0), 1, 10000, &pingWarningTimeout) != 0)
                {
                strError = "Incorrect Ping_warning_timeout value: \'" + string(node->getValue(0)) + "\'";
                return -1;
                }
            }
        else
            {
            strError = "\'Ping_warning_timeout\' parameter value missed";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Ping_timeout") == 0)
        {
        if (node->getValue(0))
            {
            if (ParseIntInRange(node->getValue(0), 1, 10000, &pingTimeout) != 0)
                {
                strError = "Incorrect Ping_timeout value: \'" + string(node->getValue(0)) + "\'";
                return -1;
                }
            }
        else
            {
            strError = "\'Ping_timeout\' parameter value missed";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Update_web_page_period") == 0)
        {
        if (node->getValue(0))
            {
            if (ParseIntInRange(node->getValue(0), 1, 10000, &updateWebPagePeriod) != 0)
                {
                strError = "Incorrect Update_web_page_period value: \'" + string(node->getValue(0)) + "\'";
                return -1;
                }
            }
        else
            {
            strError = "\'Update_web_page_period\' parameter value missed";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Pinger") == 0)
        {
        PingerParams pp;

        if (node->getValue(0))
            {
            pp.ip = inet_addr(node->getValue(0));
            ipAddr = node->getValue(0);
            if (pp.ip == INADDR_NONE)
                {
                strError = "Incorrect IP address";
                return -1;
                }
            }
        else
            {
            strError = "Missed IP for pinger";
            return -1;
            }

        pp.bitPapams.resize(8);

        int bitsNum;
        bitOreder.clear();

        if (ParsePingerParams(node->getChildNode(), &pp, &bitsNum) < 0)
            {
            return -1;
            }

        //printf("bitsNum = %d\n", bitsNum);

        pingerParams.push_back(pp);

        if (bitsNum != 8)
            {
            strError = "Description of all bits must be present";
            return -1;
            }

        if (bitsNum == 8 && *bitOreder.begin() != 0 && *bitOreder.rbegin() != 7)
            {
            strError = "Incorrect order for pinger with IP \'" + ipAddr + "\'. Order should be between 0 and 7. Same oredr should not be used twice";
            return -1;
            }
        }

    node = node->getNextNode();
    }
return 0;
}
//-----------------------------------------------------------------------------
int SETTINGS::ParsePingerParams(const DOTCONFDocumentNode * node, PingerParams * pp, int * bitsNum)
{
*bitsNum = 0;
while (node)
    {
    if (strcasecmp(node->getName(), "name") == 0)
        {
        if (node->getValue(0))
            pp->name = node->getValue(0);
        }

    if (strcasecmp(node->getName(), "Address") == 0)
        {
        if (node->getValue(0))
            pp->address = node->getValue(0);
        }

    if (strcasecmp(node->getName(), "Description") == 0)
        {
        if (node->getValue(0))
            pp->description = node->getValue(0);
        }

    if (strcasecmp(node->getName(), "Temperature") == 0)
        {
        if (ParsePingerTemperature(node->getChildNode(), &pp->temp) < 0)
            {
            return -1;
            }
        }

    for (int i = 0; i < 8; i++)
        {
        char str[10];
        sprintf(str, "Bit%d", i);
        if (strcasecmp(node->getName(), str) == 0)
            {
            (*bitsNum)++;
            if (node->getValue(0))
                pp->bitPapams[i].name = node->getValue(0);

            if (ParseBitParams(node->getChildNode(), &pp->bitPapams[i]) < 0)
                {
                return -1;
                }

            bitOreder.insert(pp->bitPapams[i].order);
            }
        }

    node = node->getNextNode();
    }

return 0;
}
//-----------------------------------------------------------------------------
int SETTINGS::ParsePingerTemperature(const DOTCONFDocumentNode * node, PingerTemp * pt)
{
while (node)
    {
    if (strcasecmp(node->getName(), "Disabled") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'Disabled\' parameter value missed";
            return -1;
            }
        if (ParseYesNo(node->getValue(0), &pt->disabled) < 0)
            {
            strError = "Parameter \'Disabled\' has incorrect value";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "MaxTemp") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'MaxTemp\' parameter value missed";
            return -1;
            }
        if (ParseDoubleInRange(node->getValue(0), -273, 1000, &pt->maxTemp) < 0)
            {
            strError = "Parameter \'MaxTemp\' has incorrect value";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "MinTemp") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'MinTemp\' parameter value missed";
            return -1;
            }
        if (ParseDoubleInRange(node->getValue(0), -273, 1000, &pt->minTemp) < 0)
            {
            strError = "Parameter \'MinTemp\' has incorrect value";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "k") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'k\' parameter value missed";
            return -1;
            }
        if (ParseDoubleInRange(node->getValue(0), 0, 100000, &pt->k) < 0)
            {
            strError = "Parameter \'k\' has incorrect value";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "b") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'b\' parameter value missed";
            return -1;
            }
        if (ParseDoubleInRange(node->getValue(0), -100000, 100000, &pt->b) < 0)
            {
            strError = "Parameter \'b\' has incorrect value";
            return -1;
            }
        }

    node = node->getNextNode();
    }
return 0;
}
//-----------------------------------------------------------------------------
int SETTINGS::ParseBitParams(const DOTCONFDocumentNode * node, BitParam * bp)
{
while (node)
    {
    if (strcasecmp(node->getName(), "Normal_value") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'Normal_value\' parameter value missed";
            return -1;
            }
        if (ParseIntInRange(node->getValue(0), 0, 1, &bp->normalValue) < 0)
            {
            strError = "Parameter \'Normal_value\' has incorrect value";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Disabled") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'Disabled\' parameter value missed";
            return -1;
            }
        if (ParseYesNo(node->getValue(0), &bp->disabled) < 0)
            {
            strError = "Parameter \'Disabled\' has incorrect value";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "Order") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'Order\' parameter value missed";
            return -1;
            }
        if (ParseIntInRange(node->getValue(0), 0, 7, &bp->order) < 0)
            {
            strError = "Parameter \'Order\' has incorrect value";
            return -1;
            }
        }

    if (strcasecmp(node->getName(), "name_bad") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'name_bad\' parameter value missed";
            return -1;
            }
        bp->name_0 = node->getValue(0);
        }

    if (strcasecmp(node->getName(), "name_norm") == 0)
        {
        if (!node->getValue(0))
            {
            strError = "\'name_norm\' parameter value missed";
            return -1;
            }
        bp->name_1 = node->getValue(0);
        }

    node = node->getNextNode();
    }
return 0;
}
//-----------------------------------------------------------------------------
const string & SETTINGS::GetScriptName() const
{
return scriptName;
}
//-----------------------------------------------------------------------------
const string & SETTINGS::GetLogName() const
{
return logName;
}
//-----------------------------------------------------------------------------
int SETTINGS::GetPingWarningTimeout() const
{
return pingWarningTimeout;
}
//-----------------------------------------------------------------------------
int SETTINGS::GetPingTimeout() const
{
return pingTimeout;
}
//-----------------------------------------------------------------------------
int SETTINGS::GetUpdateWebPagePeriod() const
{
return updateWebPagePeriod;
}
//-----------------------------------------------------------------------------
int SETTINGS::GetPingerCount() const
{
return pingerParams.size();
}
//-----------------------------------------------------------------------------
int SETTINGS::GetPingerParams(int num, PingerParams * pingerParam) const
{
if (num < 0 || num >= (int)pingerParams.size())
    {
    strError = "Requested incorrect pinger";
    return -1;
    }
*pingerParam = pingerParams[num];
return 0;
}
//-----------------------------------------------------------------------------
void SETTINGS::Print() const
{
cout << "configFileName = " << configFileName << endl;
cout << "scriptName = " << scriptName << endl;
cout << "pingWarningTimeout = " << pingWarningTimeout << endl;
cout << "pingTimeout = " << pingTimeout << endl;
cout << "updateWebPagePeriod = " << updateWebPagePeriod << endl;
cout << "pingerParams.size() = " << pingerParams.size() << endl;

for (unsigned i = 0; i < pingerParams.size(); i++)
    {

    cout << "----------------------------------" << endl;

    cout << "ip = " << pingerParams[i].ip << endl;
    cout << "name = " << pingerParams[i].name << endl;
    cout << "address = " << pingerParams[i].address << endl;

    cout << "k = " << pingerParams[i].temp.k << endl;
    cout << "b = " << pingerParams[i].temp.b << endl;
    cout << "minTemp = " << pingerParams[i].temp.minTemp << endl;
    cout << "maxTemp = " << pingerParams[i].temp.maxTemp << endl;
    cout << "Disabled = " << pingerParams[i].temp.disabled << endl;

    for (unsigned j = 0; j < 8; j++)
        {
        cout << "Bit" << j << endl;
        cout << "Order = " << pingerParams[i].bitPapams[j].order << endl;
        cout << "Normal_value = " << pingerParams[i].bitPapams[j].normalValue << endl;
        cout << "name = " << pingerParams[i].bitPapams[j].name << endl;
        cout << "name_0 = " << pingerParams[i].bitPapams[j].name_0 << endl;
        cout << "name_1 = " << pingerParams[i].bitPapams[j].name_1 << endl;
        }
    }

cout << "----------------------------------" << endl;

return;
}
//-----------------------------------------------------------------------------
const string & SETTINGS::GetHTMLEncoding() const
{
return HTMLEncoding;
}
//-----------------------------------------------------------------------------
const std::string & SETTINGS::GetTplFileName() const
{
return tplFileName;
}
//-----------------------------------------------------------------------------
const std::string & SETTINGS::GetOutputFileName() const
{
return outputFileName;
}
//-----------------------------------------------------------------------------
