#ifndef readconfh_H
#define readconfh_H

#include <string>
#include <vector>
#include <set>

#include "common_settings.h"


//-----------------------------------------------------------------------------
struct PingerTemp
{
    PingerTemp()
    {
        minTemp = 0;
        maxTemp = 255;
        k = 1;
        b = 0;
        disabled = false;
    };
    double  minTemp;
    double  maxTemp;
    double  k;
    double  b;
    bool    disabled;
};
//-----------------------------------------------------------------------------
struct BitParam
{
    BitParam()
    {
        normalValue = 0;
        name_0 = "0";
        name_1 = "1";
        disabled = false;
    };
    std::string     name;
    std::string     name_0;
    std::string     name_1;
    int             order;
    bool            disabled;
    int             normalValue;
};
//-----------------------------------------------------------------------------
struct PingerParams
{
    PingerParams()
        {
        BitParam bp;
        for (int i = 0; i < 8; i++)
            bitPapams.push_back(bp);
        }
    std::string             name;
    uint32_t                ip;
    std::string             address;
    std::string             description;
    PingerTemp              temp;
    std::vector<BitParam>   bitPapams;
};
//-----------------------------------------------------------------------------
class SETTINGS: public COMMON_SETTINGS
{
public:
    SETTINGS(const string & configFileName);
    ~SETTINGS();
    virtual int             ReadSettings();

    const std::string     & GetScriptName() const;
    const std::string     & GetLogName() const;
    const std::string     & GetTplFileName() const;
    const std::string     & GetOutputFileName() const;
    const std::string     & GetHTMLEncoding() const;
    int                     GetPingWarningTimeout() const;
    int                     GetPingTimeout() const;
    int                     GetUpdateWebPagePeriod() const;
    int                     GetPingerCount() const;
    int                     GetPingerParams(int num, PingerParams * pingerParams) const;

    void                    Print() const;

protected:

    int                     ParsePingerParams(const DOTCONFDocumentNode * node, PingerParams * pp, int * bitsNum);
    int                     ParsePingerTemperature(const DOTCONFDocumentNode * node, PingerTemp * pt);
    int                     ParseBitParams(const DOTCONFDocumentNode * node, BitParam * bp);

    std::string             configFileName;
    std::string             scriptName;
    std::string             logName;
    std::string             HTMLEncoding;
    std::string             tplFileName;
    std::string             outputFileName;
    int                     pingWarningTimeout;
    int                     pingTimeout;
    int                     updateWebPagePeriod;
    std::vector<PingerParams> pingerParams;

    set<int>                bitOreder;
};
//-----------------------------------------------------------------------------

#endif

